 /*
 * file      : src/draw.c
 * project   : tm
 * with      : Gtk-2
 *
 * copyright : (C) 2012 by Claude BULIN
 *
 * TM - Edition des heures de travail / Edition of working hours
 * GNU General Public License
 * 
 * This file is part of TM.
 * 
 * TM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * at your option) any later version.
 * 
 * TM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with TM.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifdef HAVE_CONFIG_H
	#include "../config.h"
#endif

#ifdef ENABLE_NLS
	#include <libintl.h>
	#define _(String) gettext (String)
	#define gettext_noop(x) (x) 
#endif

#include <gtk/gtk.h>
#include <cairo.h>
#include <cairo-pdf.h>
#include <cairo-ps.h>
#include <cairo-xlib.h>
#include <X11/Xlib.h>
#include <string.h>
#include <stdlib.h>

#ifdef ENABLE_STATIC_LIBRARY
	#include "../lib_tm/lib.h"
#endif

#include "global.h"
#include "callbacks.h"



// 
// FORCE LE REDRAW DE LA SURFACE DE DESSIN
// 
void draw_year( void )
{
	if( NULL != VarDraw.AdrDrawingarea ) {
		gtk_widget_queue_draw( VarDraw.AdrDrawingarea );
	}
}
// 
// FORCE LA DIMENSION DE LA SURFACE DE DESSIN
// 
void draw_set_size_request( void )
{
	gtk_widget_set_size_request(
			VarDraw.AdrDrawingarea,
			(gint)((gdouble)VarDraw.image_width * VarDraw.scale),
			(gint)((gdouble)VarDraw.image_height * VarDraw.scale) );
}
// 
// AFFICHAGE DU POURCENTAGE DU ZOOM
// PARAM:
//	gdouble p_zoom_scale:	0.2 .. 6.0
// 
void draw_print_zoom_changed( gdouble p_zoom_scale )
{
	gchar	*Str = NULL;
	
	Str = g_strdup_printf( "  %3.0f%%  ", 100.0 * p_zoom_scale );
	gtk_label_set_text( GTK_LABEL(GLADE_GET_OBJECT("label_zoom")), Str );
	g_free( Str );
	Str = NULL;
}
// 
// RETOURNE LE POURCENTAGE DU ZOOM EN FONCTION DE LA TAILLE DE LA FENETRE
// PARAM:
//	gdouble p_zoom_scale:	0.2 .. 6.0
// RETURN:
//	gdouble percent_zoom
// 
gdouble draw_get_scale_adjust( void )
{
	gdouble	return_scale = 1.0;
	
	if( TRUE == VarDraw.BoolScaleAdjust ) {
		gdouble	scale_width;
		gdouble	scale_height;
		// La definition SCALE_MIN correspond au pourcentage de representation le plus petit de l'image de fond
		// SCALE_MIN doit etre adaptee suivant la taille la plus petite de la fenetre
		if( VarDraw.scale > SCALE_MIN ) {
			VarDraw.scale = SCALE_MIN;
			draw_set_size_request();
			// while (gtk_events_pending()) gtk_main_iteration();
		}
		scale_width  = (gdouble)VarDraw.AdrDrawingarea->allocation.width / (gdouble)VarDraw.image_width;
		scale_height = (gdouble)VarDraw.AdrDrawingarea->allocation.height / (gdouble)VarDraw.image_height;
		
		return_scale = MIN(scale_width, scale_height );
	}
	else {
		return_scale = VarDraw.scale;
	}
	draw_print_zoom_changed( return_scale );
	return( return_scale );
}
// 
// 
// 
// 
void draw_test( cairo_t *p_cr  )
{
	gint		Line;
	gint		LigneEnCours;
	gchar		*StrLine = NULL;
	gdouble		PosY;
	gdouble		h;
	gdouble		Page = 0;
	
	cairo_set_source_rgb( p_cr, 0.0, 0.0, 1.0 );
	
	h = _SIZE_ONE_PAGE_;
	PosY = _POS_LINE_Y_BEGIN_PAGE_;
	
	for( LigneEnCours = Line = 1; PosY < VarDraw.image_height; Line ++) {
		
		// SI PREMIERE LIGNE, AFFICHAGE NUMERO DE LA PAGE EN HAUT
		if( PosY == (Page * h) + _POS_LINE_Y_BEGIN_PAGE_ ) {
			StrLine = g_strdup_printf( "Page en haut %d", (gint)Page +1 );
			g_print(" %s \n", StrLine );
			cairo_move_to( p_cr, 90., (gint)((Page) * h) + _POS_LINE_Y_BEGIN_PAGE_ );
			cairo_show_text( p_cr, StrLine );
			g_free( StrLine );
			StrLine = NULL;
		}

		// AFFICHAGE DE LA LIGNE EN COURS
		StrLine = g_strdup_printf( "%02d : %03d", LigneEnCours, Line );
		cairo_move_to( p_cr, 40., PosY );
		cairo_show_text( p_cr, StrLine );
		g_free( StrLine );
		StrLine = NULL;
			
		// LIGNE SUIVANTE	
		PosY +=  _POS_LINE_Y_;
		
		// SI FIN DE PAGE ATTEINTE
		if( PosY + _POS_LINE_Y_ > (Page +1) * h ) {
			
			// SI DERNIERE LIGNE, AFFICHAGE NUMERO DE LA PAGE EN BAS
			StrLine = g_strdup_printf( "Page en bas %d", (gint)Page +1 );
			g_print(" %s \n", StrLine );
			cairo_move_to( p_cr, 90., PosY -_POS_LINE_Y_ );
			cairo_show_text( p_cr, StrLine );
			g_free( StrLine );
			StrLine = NULL;
			
			Page ++;
			PosY  = (Page * h);
			PosY += _POS_LINE_Y_BEGIN_PAGE_;
			LigneEnCours = 1;
		// SINON
		} else
			LigneEnCours ++;
		
	}
}
// 
// 
void draw_page( cairo_t *p_cr )
{
	GList			*ListPages = NULL;
	LINE			*Line = NULL;
	GList			*ListLines = NULL;
	LINE_OF_PAGE		*Details = NULL;
	cairo_text_extents_t	extents;
	
	// draw_test( p_cr );
	
	if( NULL != AllPages.Page ) {
		
		// CLIPPING DE LA ZONE POUR NE PAS DEBORDER A DROITE
		cairo_rectangle( p_cr,
				 0,
				 0,
				 VarDraw.image_width,
				 VarDraw.image_height
				 );	
		cairo_clip( p_cr );
		cairo_new_path( p_cr );
		
		ListPages = g_list_first( AllPages.Page );
		while( ListPages ) {
			if( NULL != (Line = (LINE *)ListPages->data)) {
				ListLines = g_list_first( Line->LineDetail );
				while( ListLines ) {
					if( NULL != (Details = (LINE_OF_PAGE *)ListLines->data) ) {
						// *********************************************************
						// AFFICHAGE
						// *********************************************************
						if( NOUVELLE_PAGE == Details->LineType ||
							STRING == Details->LineType ||
							STRING_REPAIR == Details->LineType ||
							STRING_PAGE == Details->LineType ||
							STRING_COMMENT == Details->LineType ) {
							
							cairo_set_font_size( p_cr, Details->SizeFont );
							
							if( COLOR_BLACK == Details->TypeColor )
								cairo_set_source_rgb( p_cr, 0.0, 0.0, 0.0 );
							else	cairo_set_source_rgb( p_cr, 1.0, 0.0, 0.0 );

							cairo_select_font_face(
								p_cr,
								VarDraw.FontName,
								Details->FontSlantType == FONT_SLANT_NORMAL ? CAIRO_FONT_SLANT_NORMAL : CAIRO_FONT_SLANT_ITALIC,
								Details->FontWeightType == FONT_WEIGHT_NORMAL ? CAIRO_FONT_WEIGHT_NORMAL : CAIRO_FONT_WEIGHT_BOLD
								);
							
							// CALCUL LONGUEURE DE CHAINE
							cairo_text_extents( p_cr, Details->Str, &extents );
							Details->LenStr = extents.width;
							
							// RECUP. DU PLUS LONG COMMENTAIRE
							if( STRING_COMMENT == Details->LineType ) {
								// Details->PosX = VarDraw.image_width - Details->LenStr;
								// Details->PosX -= 55.0;
							}
							// FIXE LE NUMERO DE PAGE AU CENTRE
							else if( STRING_PAGE == Details->LineType ) {
								Details->PosX = VarDraw.image_width / 2;
								Details->PosX -= Details->LenStr / 2.;
								Details->PosXPS = 596 / 2;
								Details->PosXPS -= Details->LenStr / 2.;
								Details->PosXPS -= 55 / 2;
							}
							cairo_move_to( p_cr, Details->PosX, Details->PosY );
							cairo_show_text( p_cr, Details->Str );
						}
						if( LINE_GRAPH == Details->LineType ) {
							cairo_set_line_width( p_cr, 1.5 );
							cairo_move_to( p_cr, Details->LinePosX, Details->LinePosY );
							cairo_rel_line_to( p_cr, Details->LineLen, 0.0 );
							cairo_stroke ( p_cr );
						}
						// *********************************************************
					}
					ListLines = g_list_next( ListLines );
				}
			}
			ListPages = g_list_next( ListPages );
		}
	}
}
// 
// AFFICHAGE DE LA ZONE DE DESSIN
// PARAM:
// RETURN:
// 
void draw_paint( cairo_surface_t *p_cs )
{
	cairo_t			*cr;
	gint			canvas_w, canvas_h;
	gint			Page;
	gdouble			h;
	gdouble			PosY;
	
	if( FALSE == VarDraw.BoolSaveToFile ) {
		cr = gdk_cairo_create( VarDraw.AdrDrawingarea->window );
	}
	else {
		cr = cairo_create( p_cs );
	}
	cairo_save( cr );
	VarDraw.scale = draw_get_scale_adjust();
	canvas_w = VarDraw.AdrDrawingarea->allocation.width;
	canvas_h = VarDraw.AdrDrawingarea->allocation.height;
	VarDraw.x0 = ((canvas_w / VarDraw.scale) - VarDraw.image_width) / 2.0;
	VarDraw.y0 = ((canvas_h / VarDraw.scale) - VarDraw.image_height) / 2.0;

	cairo_scale( cr, VarDraw.scale, VarDraw.scale );
	cairo_translate( cr, VarDraw.x0, VarDraw.y0 );
	cairo_set_source_surface( cr, VarDraw.image, 0.0, 0.0 );
	cairo_paint( cr );	
	
	h = ((gdouble)VarDraw.image_height / (gdouble)VarDraw.TotalPages);
	
	// PAINT PREVIEW SURFACE: BLACK
	cairo_set_line_width( cr, 0.5 );
	for( Page = 0; Page < VarDraw.TotalPages; Page ++ ) {
		
		PosY = Page * h;
		
		// PAINT PREVIEW SURFACE: BLACK
		cairo_set_source_rgb( cr, 0.0, 0.0, 0.0 );
		cairo_rectangle( cr,
				 3,
				 PosY +3,
				 VarDraw.image_width,
				 h -5
				 );	
		cairo_fill ( cr );

		// PAINT PREVIEW SURFACE: WHITE
		cairo_set_source_rgb( cr, 1.0, 1.0, 1.0 );
		cairo_rectangle( cr,
				 0,
				 PosY +2,
				 VarDraw.image_width,
				 h -7
				 );	
		cairo_fill ( cr );
		
		// BORDER LINE AROUND: BLACK
		cairo_set_source_rgb( cr, 0.0, 0.0, 0.0 );
		cairo_rectangle( cr,
				 0,
				 PosY +2,
				 VarDraw.image_width,
				 h -7
				 );	
		cairo_stroke( cr );
		
	}
	
	// AFFICHAGE
	cairo_select_font_face( cr, VarDraw.FontName, CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL );
	cairo_set_font_size( cr, (gdouble)VarDraw.SizeFontName );
	cairo_set_source_rgb( cr, 0.0, 0.0, 0.0 );
	
	draw_page( cr  );
	
	cairo_restore (cr);
	cairo_destroy( cr );
}




















